
#include "datatypes.h"
#include "can.h"
#include "can_user.h"
#include "hardware.h"
#include "crc_data.h"
#include "systime.h"
#include "serial.h"
#include "ser_user.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// size for user text buffer received from partner like terminal program or other device
#define	USER_TEXT_SIZE		30


// identifier is needed by PCANFlash.exe -> do not delete
const b8_t Ident[] __attribute__ ((used)) = { "PCAN-Router-DR"};


// info data for PCANFlash.exe
const crc_array_t  C2F_Array __attribute__((section(".C2F_Info"), used)) = {

	.Str = CRC_IDENT_STRING,
	.Version = 0x21,
	.Day = 5,
	.Month = 5,
	.Year = 6,
	.Mode = 1,

	// crc infos are patched during link time by flash.ld
	// crc value is patched by PCANFlash.exe
};


// variables for LED toggle
static u8_t LED_toggleCAN1;
static u8_t LED_toggleCAN2;


// variables for serial reception
static u8_t txt_idx;
static b8_t user_text[USER_TEXT_SIZE];



// main_greeting()
// transmitt a message at module start
static void  main_greeting ( void)
{
	CANMsg_t  Msg;
	

	Msg.Id   = 0x123;
	Msg.Len  = 8;
	Msg.Type = CAN_MSG_STANDARD;
	
	Msg.Data32[0] = 0x67452301;
	Msg.Data32[1] = 0xEFCDAB89;
	
	// Send Msg
	CAN_UserWrite ( CAN_BUS1, &Msg);
}




// main()
// entry point from crt0.S
int  main ( void)
{
	// init hardware
	HW_Init();
	
	// init timer
	SYSTIME_Init();
	
	// init serial
	SER_UserInit();
	
	// wait 180 millis for CAN DC/DC converters
	SYSTIME_wait_us ( 180000);
	
	// init CAN
	CAN_UserInit();
	
	// Set green LEDs for CAN1 and CAN2
	HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
	HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
	
	// send the greeting message
	main_greeting();
	
	// main loop
	while ( 1)
	{
		CANMsg_t  RxMsg, TxMsg;
		SERStatus_t  serstat;
		u8_t  ser_read;
		

		// process messages from CAN1
		if ( CAN_UserRead ( CAN_BUS1, &RxMsg) != 0)
		{
			// message received from CAN1
			LED_toggleCAN1 ^= 1;

			if ( LED_toggleCAN1)
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN1, HW_LED_GREEN);
			}
			
			
			// catch ID 00000650h to send on serial as ascii
			if ( RxMsg.Id == 0x650  &&  RxMsg.Type == CAN_MSG_EXTENDED)
			{
				b8_t  textbuff[30];
				u32_t  textlen;
				
				
				// convert a 15-bit signal to ASCII text
				textlen = snprintf ( &textbuff[0], 30, "RPM = %d\n", RxMsg.Data16[0] & 0x7FFF);
				
				// send on serial to terminal
				SER_Write ( SER_PORT1, &textbuff[0], textlen);
			}
			
			// catch ID 611h to send on serial as binary
			else if ( RxMsg.Id == 0x611  &&  RxMsg.Type == CAN_MSG_STANDARD)
			{
				SER_Write ( SER_PORT1, &RxMsg.Data8[0], RxMsg.Len);
			}
			
			// route message to CAN2
			else
			{
				// copy message
				TxMsg.Id   = RxMsg.Id;
				TxMsg.Type = RxMsg.Type;
				TxMsg.Len  = RxMsg.Len;
			
				TxMsg.Data32[0]	= RxMsg.Data32[0];
				TxMsg.Data32[1]	= RxMsg.Data32[1];
				
				// send
				CAN_UserWrite ( CAN_BUS2, &TxMsg);
			}
		}


		// process messages from CAN2
		if ( CAN_UserRead ( CAN_BUS2, &RxMsg) != 0)
		{
			// message received from CAN2
			LED_toggleCAN2 ^= 1;

			if ( LED_toggleCAN2)
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_ORANGE);
			}
			
			else
			{
				HW_SetLED ( HW_LED_CAN2, HW_LED_GREEN);
			}
			
			
			// copy message
			TxMsg.Id   = RxMsg.Id;
			TxMsg.Type = RxMsg.Type;
			TxMsg.Len  = RxMsg.Len;
			
			TxMsg.Data32[0]	= RxMsg.Data32[0];
			TxMsg.Data32[1]	= RxMsg.Data32[1];
			
			// send
			CAN_UserWrite ( CAN_BUS1, &TxMsg);
		}
		
		
		// check input from serial. Modify code for your own needs. Check buffer for overflows if needed.
		serstat = SER_Read ( SER_PORT1, &user_text[txt_idx], USER_TEXT_SIZE - txt_idx, &ser_read);
		
		if ( serstat == SER_ERR_OK)
		{
			// data read from serial
			if ( user_text[txt_idx] == '\n')
			{
				// new line character, do nothing
			}
			
			else if ( user_text[txt_idx] == '\r')
			{
				// user string complete
				
				user_text[txt_idx] = 0;		// terminate with null
				
				if ( strncmp ( user_text, "fuel=", 5) == 0)
				{
					// new value for fuel level. Write a text like:
					// fuel=34
					// to a terminal and the router will send ID 450h  4  22h 00h 00h 00h
					TxMsg.Id   = 0x450;
					TxMsg.Type = CAN_MSG_STANDARD;
					TxMsg.Len  = 4;
					
					TxMsg.Data32[0] = strtoul ( &user_text[5], NULL, 10);
					
					// send
					CAN_UserWrite ( CAN_BUS1, &TxMsg);
				}
				
				txt_idx = 0;
			}
			
			else
			{
				// wait for more characters
				txt_idx += ser_read;
			}
		}
	}
}
